@@ -255,3 +255,26 @@ class Enough(object):
         host = clone.create_service_matching_snapshot(snapshot)
         clone.openstack.replace_volume(host, snapshot, delete_volume=True)
         return clone
+
+    class VolumeResizeUndefined(Exception):
+        pass
+
+    class VolumeResizeNoSize(Exception):
+        pass
+
+    def volume_resize(self, host, volume):
+        d = self.heat.get_stack_definitions()[host]
+        volumes = d.get('openstack_volumes')
+        if not volumes:
+            raise Enough.VolumeResizeUndefined(f'no openstack_volumes variable is set for {host}')
+        volume_size = None
+        for info in volumes:
+            if info['name'] == volume:
+                volume_size = info.get('size')
+                break
+        if volume_size is None:
+            raise Enough.VolumeResizeNoSize(f'no size found for {volume} in the openstack_volumes '
+                                            f'variable ({volumes}) for {host}')
+        r = self.openstack.volume_resize(host, volume, int(volume_size))
+        self.openstack.wait_for_ssh(d['ansible_host'], d['ansible_port'])
+        return r
