@@ -35,9 +35,21 @@ class OpenStackBase(object):
             _env={'OS_CLIENT_CONFIG_FILE': self.config_file},
         )
 
+    @staticmethod
+    @retry((socket.timeout, ConnectionRefusedError), 9)
+    def wait_for_ssh(ip, port):
+        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        s.settimeout(5)
+        s.connect((ip, int(port)))
+        line = s.makefile("rb").readline()
+        assert line.startswith(b'SSH-')
+
 
 class Stack(OpenStackBase):
 
+    class IPNotFound(Exception):
+        pass
+
     def __init__(self, config_dir, definition=None, **kwargs):
         super().__init__(config_dir, **kwargs)
         log.info(f'OS_CLIENT_CONFIG_FILE={self.config_file}')
@@ -49,7 +61,7 @@ class Stack(OpenStackBase):
     def set_public_key(self, path):
         self.public_key = open(path).read().strip()
 
-    def _create_or_update(self, action, network):
+    def _create_or_update(self, action):
         d = self.definition
         parameters = [
             f'--parameter=public_key={self.public_key}',
@@ -62,20 +74,18 @@ class Stack(OpenStackBase):
             parameters.append(f"--parameter=volume_size={d['volumes'][0]['size']}")
             parameters.append(f"--parameter=volume_name={d['volumes'][0]['name']}")
         if 'network' in d:
-            parameters.append(f'--parameter=network={network}')
+            parameters.append(f'--parameter=network={d["network"]}')
+            parameters.append(
+                f'--parameter=network_interface_unconfigured={d["network_interface_unconfigured"]}')
+            parameters.append(
+                f'--parameter=network_interface_routed={d["network_interface_routed"]}')
+            parameters.append(
+                f'--parameter=network_interface_not_routed={d["network_interface_not_routed"]}')
         log.info(f'_create_or_update: {d["name"]} {parameters}')
         self.o.stack(action, d['name'],
                      '--wait', '--timeout=600',
                      '--template', self.get_template(),
                      *parameters)
-        return self.get_output()
-
-    def get_output(self):
-        r = self.o.stack('output', 'show', '--format=value', '-c=output', '--all',
-                         self.definition['name'])
-        result = json.loads(r.stdout)
-        assert 'output_error' not in result, result['output_error']
-        return result['output_value']
 
     def list(self):
         return [
@@ -83,27 +93,39 @@ class Stack(OpenStackBase):
             self.o.stack.list('--format=value', '-c', 'Stack Name', _iter=True)
         ]
 
-    @staticmethod
-    @retry((socket.timeout, ConnectionRefusedError), 9)
-    def wait_for_ssh(ip, port):
-        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        s.settimeout(5)
-        s.connect((ip, int(port)))
-        line = s.makefile("rb").readline()
-        assert line.startswith(b'SSH-')
-
     def create_or_update(self, return_on_create=False):
         if self.definition['name'] not in self.list():
             self.create()
             if return_on_create:
                 return None
-        info = self.update()
-        if not self.only_internal() or self.args.get('route_to_internal', True):
+        else:
+            self._create_or_update('update')
+        self.create_internal_network()
+        self.connect_internal_network()
+        info = {
+            'ipv4': self.get_ipv4(),
+            'port': self.definition['port'],
+        }
+        if not self.internal_only() or self.args.get('route_to_internal', True):
             self.wait_for_ssh(info['ipv4'], info['port'])
         Hosts(self.config_dir).create_or_update(
             self.definition['name'], info['ipv4'], info['port'])
         return info
 
+    @retry((IPNotFound, sh.ErrorReturnCode_1), 5)
+    def get_ipv4(self):
+        server = self.definition['name']
+        o = OpenStack(self.config_dir, **self.args)
+        if self.internal_only():
+            ipv4 = o.server_ip_in_network(
+                server, self.definition.get('internal_network', OpenStackBase.INTERNAL_NETWORK))
+        else:
+            ipv4 = o.server_ip_in_network(
+                server, self.definition.get('network', 'Ext-Net'))
+        if ipv4 is None:
+            raise Stack.IPNotFound()
+        return ipv4
+
     def create_internal_network(self):
         network = self.definition.get('internal_network', OpenStackBase.INTERNAL_NETWORK)
         cidr = self.definition.get('internal_network_cidr', OpenStackBase.INTERNAL_NETWORK_CIDR)
@@ -118,33 +140,22 @@ class Stack(OpenStackBase):
             return
         self.o.server.add.network(server, network)
 
-    def update(self):
-        self.create_internal_network()
-        r = self._create_or_update('update', self.definition.get('network', 'Ext-Net'))
-        self.connect_internal_network()
-        return r
-
-    def only_internal(self):
-        return self.definition.get('network', 'Ext-Net') != 'Ext-Net'
+    def internal_only(self):
+        return self.definition.get('network_internal_only', False)
 
     def create(self):
         try:
-            log.info('create on network Ext-Net')
-            info = self._create_or_update('create', 'Ext-Net')
+            log.info('create or update')
+            self._create_or_update('create')
         except sh.ErrorReturnCode_1:
-            log.info('retry create on network Ext-Net')
+            log.info('retry create or update')
             # retry once because there is no way to increase the timeout and create fails often
-            info = self._create_or_update('update', 'Ext-Net')
-        if self.only_internal():
-            log.info(f'remove network Ext-Net')
-            # wait for the interface to settle before removing it
-            self.wait_for_ssh(info['ipv4'], info['port'])
-            server = self.definition['name']
-            self.o.server.remove.network(server, 'Ext-Net')
+            self._create_or_update('update')
 
     def delete_no_wait(self):
         name = self.definition['name']
         if name not in self.list():
+            Hosts(self.config_dir).delete(name)
             return False
 
         self.o.stack.delete('--yes', name)
@@ -186,11 +197,23 @@ class Heat(OpenStackBase):
 
     def get_stack_definition(self, host):
         h = self.get_stack_definitions()[host]
+        if h.get('network_internal_only', False):
+            network_interface_unconfigured = h.get('network_primary_interface', 'eth0')
+            network_interface_routed = h.get('network_secondary_interface', 'eth1')
+            network_interface_not_routed = 'noname'
+        else:
+            network_interface_unconfigured = 'noname'
+            network_interface_routed = h.get('network_primary_interface', 'eth0')
+            network_interface_not_routed = h.get('network_secondary_interface', 'eth1')
         definition = {
             'name': host,
             'port': h.get('ansible_port', '22'),
             'flavor': h.get('openstack_flavor', 's1-2'),
             'network': h.get('openstack_network', 'Ext-Net'),
+            'network_internal_only': h.get('network_internal_only', False),
+            'network_interface_unconfigured': network_interface_unconfigured,
+            'network_interface_routed': network_interface_routed,
+            'network_interface_not_routed': network_interface_not_routed,
             'internal_network': OpenStackBase.INTERNAL_NETWORK,
             'internal_network_cidr': h.get('openstack_internal_network_cidr',
                                            OpenStackBase.INTERNAL_NETWORK_CIDR),
@@ -290,6 +313,18 @@ class OpenStackShutoff(Exception):
     pass
 
 
+class OpenStackVolumeResizeMissing(Exception):
+    pass
+
+
+class OpenStackVolumeResizeMismatch(Exception):
+    pass
+
+
+class OpenStackVolumeResizeShrink(Exception):
+    pass
+
+
 class OpenStack(OpenStackBase):
 
     def __init__(self, config_dir, **kwargs):
@@ -384,7 +419,10 @@ class OpenStack(OpenStackBase):
 
     def subnet_create(self, name, cidr):
         if not self.subnet_exists(name):
-            self.o.subnet.create('--subnet-range', cidr, '--network', name, name)
+            self.o.subnet.create('--subnet-range', cidr,
+                                 '--dns-nameserver=8.8.8.8',
+                                 '--network', name,
+                                 name)
 
     def subnet_update_dns(self, name, *dns):
         args = [f'--dns-nameserver={ip}' for ip in dns]
@@ -404,7 +442,7 @@ class OpenStack(OpenStackBase):
         info = info.strip()
         if info == '':
             return None
-        pattern = r"ip_address='(.*?)'"
+        pattern = r"ip_address='([0-9.]*?)'"
         m = re.search(pattern, info)
         assert m, f're.search({pattern}, {info})'
         return m.group(1)
@@ -489,6 +527,33 @@ class OpenStack(OpenStackBase):
         self.o.server.add.volume(host, volume_id)
         self.o.server.start(host)
 
+    def volume_resize(self, host, volume, size):
+        attached = self.o.server.show(
+            '--format=value', '-c', 'volumes_attached', host).stdout.strip()
+        if len(attached) == 0:
+            raise OpenStackVolumeResizeMissing(
+                f'{host} is not attached {volume}, it is not attached to any volume')
+        current_volume_id = re.sub("id='(.*)'", "\\1", attached.decode('utf-8'))
+        current_volume = self.o.volume.show(
+            '--format=value', '-c', 'name', current_volume_id).strip()
+        if current_volume != volume:
+            raise OpenStackVolumeResizeMismatch(
+                f'{host} is not attached {volume}, it is attached to {current_volume}')
+        current_size = int(self.o.volume.show('-c', 'size', '--format=value', volume).strip())
+        if current_size == size:
+            log.info(f'resize of {host} volume {volume} is not necessary, it already is {size}GB')
+            return False
+        if current_size > size:
+            raise OpenStackVolumeResizeShrink(
+                f'{host} {volume} is {current_size}GB and cannot be shrinked to {size}GB')
+        self.o.server.stop(host)
+        self.server_wait_shutoff(host)
+        self.o.server.remove.volume(host, volume)
+        self.o.volume.set('--size', size, volume)
+        self.o.server.add.volume(host, volume)
+        self.o.server.start(host)
+        return True
+
     def region_empty(self):
         volumes = self.o.volume.list()
         servers = self.o.server.list()
