@@ -23,7 +23,7 @@ class Create(Command):
         args.update(vars(parsed_args))
         d = dotenough.DotEnoughOpenStack(settings.CONFIG_DIR, args['domain'])
         d.ensure()
-        o = OpenStack(settings.CONFIG_DIR, args['clouds'])
+        o = OpenStack(settings.CONFIG_DIR, **args)
         o.backup_create(args['volumes'])
 
 
@@ -43,5 +43,5 @@ class Prune(Command):
         args.update(vars(parsed_args))
         d = dotenough.DotEnoughOpenStack(settings.CONFIG_DIR, args['domain'])
         d.ensure()
-        o = OpenStack(settings.CONFIG_DIR, args['clouds'])
+        o = OpenStack(settings.CONFIG_DIR, **args)
         o.backup_prune(args['days'])
