@@ -29,4 +29,4 @@ def test_snapshots(host):
         print(cmd.stderr)
         assert 0 == cmd.rc
 
-        assert openstack(host, "snapshot list -f value -c Name") == ""
+        assert openstack(host, "volume snapshot list -f value -c Name") == ""
