@@ -10,6 +10,9 @@ def test_bind_external(host):
     domain = bind_host.run("hostname -d").stdout.strip()
     address = bind_host.ansible.get_variables()['ansible_host']
 
+    with host.sudo():
+        host.run("apt-get install -y dnsutils")
+
     infrastructure = get_driver()
 
     if infrastructure == 'openstack':
