@@ -60,18 +60,6 @@ def get_hosts(session, enough):
     return (hosts != updated, updated)
 
 
-def reboot_internal(enough, hosts):
-    for host in enough.service.hosts_with_internal_network(hosts):
-        #
-        # The reboot cannot be done immediately after removing the interface
-        # for some reason. Otherwise sshd does not bind to the interface.
-        #
-        # Hopefully doing it at this stage is late enough and does not cause this
-        # problem.
-        #
-        enough.openstack.o.server.reboot('--hard', host)
-
-
 def pytest_sessionstart(session):
     service_directory = session.config.getoption("--enough-service")
     domain = f'{service_directory}.test'
@@ -92,12 +80,11 @@ def pytest_sessionstart(session):
     if needs_vpn:
         if not e.vpn_has_credentials():
             vpn_hosts = ','.join(e.service.service2hosts['openvpn'])
-            reboot_internal(e, hosts)
             e.playbook.run([
                 '--private-key', f'{e.config_dir}/infrastructure_key',
                 '-i', f'playbooks/openvpn/inventory',
                 f'--limit={vpn_hosts},localhost',
-                'playbooks/openvpn/playbook.yml',
+                'playbooks/openvpn/conftest-playbook.yml',
             ])
         e.vpn_connect()
 
