@@ -5,6 +5,7 @@ import requests_mock as requests_mock_module
 from tests.modified_environ import modified_environ
 
 from enough import settings
+from enough.common import openstack
 from enough.common.openstack import Stack, Heat, OpenStack
 
 
@@ -13,30 +14,7 @@ from enough.common.openstack import Stack, Heat, OpenStack
 #
 @pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
                     reason='skip integration test')
-def test_stack_create_or_update_default(openstack_name):
-    d = {
-        'name': openstack_name,
-        'flavor': 's1-2',
-        'port': '22',
-        'volumes': [
-            {
-                'size': '1',
-                'name': openstack_name,
-            },
-        ],
-    }
-    s = Stack(settings.CONFIG_DIR, d)
-    s.set_public_key('infrastructure_key.pub')
-    r = s.create_or_update()
-    assert r['port'] == '22'
-    assert 'ipv4' in r
-    assert r == s.create_or_update()
-    s.delete()
-
-
-@pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
-                    reason='skip integration test')
-def test_stack_create_or_update_with_internal_network(openstack_name):
+def test_stack_create_or_update(openstack_name):
     name = openstack_name
     network = openstack_name
     class_c = '10.101.30'
@@ -194,3 +172,60 @@ def test_openstack_replace_volume(openstack_name):
         '-f=value', '-c=Name', '--name', backup_volume).strip() == backup_volume
     assert o.o.volume.list(
         '-f=value', '-c=Name', '--name', openstack_name).strip() == openstack_name
+
+
+@pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
+                    reason='skip integration test')
+def test_openstack_volume_resize_ok(openstack_name):
+    size = 1
+    d = {
+        'name': openstack_name,
+        'flavor': 's1-2',
+        'port': '22',
+        'volumes': [
+            {
+                'size': str(size),
+                'name': openstack_name,
+            },
+        ],
+    }
+    s = Stack(settings.CONFIG_DIR, d)
+    s.set_public_key('infrastructure_key.pub')
+    s.create_or_update()
+
+    o = OpenStack(settings.CONFIG_DIR)
+
+    assert size == int(o.o.volume.show(
+        '-c', 'size', '--format=value', openstack_name).strip())
+
+    new_size = 2
+    assert o.volume_resize(openstack_name, openstack_name, new_size) is True
+
+    assert new_size == int(o.o.volume.show(
+        '-c', 'size', '--format=value', openstack_name).strip())
+
+    assert o.volume_resize(openstack_name, openstack_name, new_size) is False
+
+    with pytest.raises(openstack.OpenStackVolumeResizeMismatch):
+        o.volume_resize(openstack_name, 'UNKNOWN', new_size)
+
+    with pytest.raises(openstack.OpenStackVolumeResizeShrink):
+        o.volume_resize(openstack_name, openstack_name, 1)
+
+
+@pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
+                    reason='skip integration test')
+def test_openstack_volume_resize_no_volume(openstack_name):
+    d = {
+        'name': openstack_name,
+        'flavor': 's1-2',
+        'port': '22',
+    }
+    s = Stack(settings.CONFIG_DIR, d)
+    s.set_public_key('infrastructure_key.pub')
+    s.create_or_update()
+
+    o = OpenStack(settings.CONFIG_DIR)
+
+    with pytest.raises(openstack.OpenStackVolumeResizeMissing):
+        o.volume_resize(openstack_name, openstack_name, 1)
